
from __future__ import unicode_literals
import os
import sys
import re
import conf
import rstinfo
import shutil

__version__ = "1.0.0"


TYPE_CLASS = 1
TYPE_UNION = 2
TYPE_FUNCTION = 3
TYPE_ENUM = 4
TYPE_DEFINE = 5
TYPE_TYPEDEFINE = 6
TYPE_VARIABLE = 7
struct_filename_dic = {}
en_title = {
        '结构体':'Struct',
        '枚举':'Enum',
        '类型定义':'Typedef',
        '宏定义':'Define',
        '函数列表':'Function List'
        }

def getdatastructbyfilename(filename, dtype):
    if not os.path.exists(filename):
        return None
    dname = None
    with open(filename,'r') as fd:
        lines = fd.readlines()
        for line in lines:
            #line = line.strip()
            if not line.strip():
                continue
            dname = re.sub(dtype + ' *(.+)', '\g<1>', line[:-1])
            if dname != line[:-1]:
                break
    return dname

def rst_file_process(rstfile, rstrootdir):
    info_dic = {'struct':[], 'function':[], 'union':[], 'enum':[], 'define':[], 'typedef':[], 'variable':[]}
    if not os.path.exists(rstfile):
        return None
    lines = []
    with open(rstfile,'r') as fd:
        lines = fd.readlines()
        #print(lines)
    datatype = 0
    for line in lines:
        if not line.strip():
            continue
        line = line.strip()
        if 'Classes' == line:
            datatype = TYPE_CLASS
            continue
        if 'Functions' == line:
            datatype = TYPE_FUNCTION
            continue
        if 'Unions' == line:
            datatype = TYPE_UNION
            continue
        if 'Enums' == line:
            datatype = TYPE_ENUM
            continue
        if 'Defines' == line:
            datatype = TYPE_DEFINE
            continue
        if 'Typedefs' == line:
            datatype = TYPE_TYPEDEFINE
            continue
        if 'Variables' == line:
            datatype = TYPE_VARIABLE
            continue
        if '- :ref:`' not in line:
            continue
        rstfilename = re.sub('- :ref:`exhale_(.+)`', '\g<1>.rst', line)
        rstfilename = os.path.join(rstrootdir, rstfilename)
        dname = 'unknow'
        #print(rstfilename, datatype)
        if datatype == TYPE_CLASS:
            rstfilename = re.sub('struct_struct', 'struct', rstfilename)
            dname = getdatastructbyfilename(rstfilename, 'Struct')
            info_dic['struct'].append({dname:rstfilename})
        if datatype == TYPE_FUNCTION:
            dname = getdatastructbyfilename(rstfilename, 'Function')
            info_dic['function'].append({dname:rstfilename})
        if datatype == TYPE_UNION:
            rstfilename = re.sub('union_union', 'union', rstfilename)
            dname = getdatastructbyfilename(rstfilename, 'Union')
            info_dic['union'].append({dname:rstfilename})
        if datatype == TYPE_ENUM:
            dname = getdatastructbyfilename(rstfilename, 'Enum')
            info_dic['enum'].append({dname:rstfilename})
        if datatype == TYPE_DEFINE:
            dname = getdatastructbyfilename(rstfilename, 'Define')
            info_dic['define'].append({dname:rstfilename})
        if datatype == TYPE_TYPEDEFINE:
            dname = getdatastructbyfilename(rstfilename, 'Typedef')
            info_dic['typedef'].append({dname:rstfilename})
        if datatype == TYPE_VARIABLE:
            dname = getdatastructbyfilename(rstfilename, 'Variable')
            info_dic['variable'].append({dname:rstfilename})
        struct_filename_dic[dname] = rstfilename
    return info_dic

def getheadfile_datastructinfo(rootrstdir):
    rst_files = os.listdir(rootrstdir)
    rst_h_files_dic = {}
    for filename in rst_files:
        if 'file_' in filename and 'program_listing_file__' not in filename  and 'file_view_hierarchy' not in filename:
            #rst_h_files.append(filename)
            rstpath = os.path.join(rootrstdir, filename)
            info_dic = rst_file_process(rstpath, rootrstdir)
            rst_h_files_dic[filename] = info_dic
    return rst_h_files_dic

def genmoduleindex(project_info,  data_struct_dic):
    curdir = os.getcwd()
    modules_dic = {}
    for key, values in project_info.items():
        modules_dic[key] = {}
        for value in values:
            value = os.path.relpath(value, curdir)
            file_rst = re.sub('/', '_', value)
            file_rst = 'file__output_' + file_rst + '.rst'
            if file_rst in data_struct_dic.keys():
                modules_dic[key][value] = data_struct_dic[file_rst]
    return modules_dic

def getindexmdbymoduledic(modules_dic, rootrstdir):
    typeslist = ['class','function','enum','union','define','typedef','variable']
    modules = {}
    for modulename, moduledic in modules_dic.items():
        #modulerstfilename = module
        modules[modulename] = {'struct':'# Structs List\n',\
                'function':'# Functions List\n','enum':'Enums List\n', \
                'union':'#Unions List\n','define':'# Defines List\n', \
                'typedef':'Typedefs List\n', 'variable':'Variables List\n'}
        for hfile, data_info_dic in moduledic.items():
            for dtype, datalist in data_info_dic.items():
                if not datalist:
                    continue
                for data in datalist:
                    for key, value in data.items():
                        mddata = '  * [' + key + '](' + os.path.basename(value) + ')\n'
                        modules[modulename][dtype] = modules[modulename][dtype] + mddata
    for modulename, data_dic in modules.items():
        for dtype, data in data_dic.items():
            mdfilename = modulename + '_' + dtype + '.md'
            mdfilename = os.path.join(rootrstdir, mdfilename)
            with open(mdfilename, 'w') as fd:
                fd.write(data)
            if conf.one_page_one_api == 'yes':
                genrst_one_api(os.path.abspath(mdfilename))
            else:
                genrst_all_api(os.path.abspath(mdfilename))

def genrst_all_api(absfilename):
    print('genrst:', absfilename)
    md_file = absfilename
    tmp_name = absfilename[:-3]
    rst_file = tmp_name + '.rst'
    fd = open(md_file, 'r')
    fr = open(rst_file, 'w')
    if tmp_name.endswith('struct'):
        title = '结构体'
    elif tmp_name.endswith('enum'):
        title = '枚举'
    elif tmp_name.endswith('typedef'):
        title = '类型定义'
    elif tmp_name.endswith('define'):
        title = '宏定义'
    elif tmp_name.endswith('function'):
        title = '函数列表'
    else:
        fr.close()
        fd.close()
        return
    if conf.language == 'en':
        title = en_title[title]
    fr.write(title + '\n#####################\n\n')
    module_part = os.path.basename(absfilename).split('_')[0]
    file_dir = os.path.dirname(absfilename)
    lib_dir = os.path.join(file_dir, 'lib')
    if not os.path.exists(lib_dir):
        os.mkdir(lib_dir)
    while True:
        line = fd.readline()
        if not line:
            break
        line = line.strip('\n')
        if line.endswith('.rst)'):
            line_name = line.split('](')[0].split('[')[1]#函数名
            line_file = line.split('](')[1].strip(')')#对应的rst文件
            struct_filename_dic[line_name] = 'exhalerst/lib/' + line_file
            fr.write('.. include:: ' + 'lib/' + line_file + '\n')
            old_path = os.path.join(file_dir, line_file)
            new_path = os.path.join(lib_dir, line_file)
            os.system("sed -i 's/Function Documentation//g' " + old_path)
            os.system("sed -i 's/Typedef Documentation//g' " + old_path)
            os.system("sed -i 's/Define Documentation//g' " + old_path)
            os.system("sed -i 's/Enum Documentation//g' " + old_path)
            os.system("sed -i 's/Struct Documentation//g' " + old_path)
            shutil.copy(old_path, new_path)
    fd.close()
    fr.close()
    data_rst = os.path.join(file_dir, module_part + '_annotated.rst')
    if conf.language == 'en':
        with open(data_rst, 'w') as fa:
            fa.write('Data Structure\n===================\n\n')
            fa.write('.. toctree::\n   :titlesonly:\n\n')
            fa.write('   Define<'+ module_part + '_define.rst>\n')
            fa.write('   Typedef<'+ module_part + '_typedef.rst>\n')
            fa.write('   Enum<'+ module_part + '_enum.rst>\n')
            fa.write('   Struct<' + module_part + '_struct.rst>\n')
    else:
        with open(data_rst, 'w') as fa:
            fa.write('数据结构\n===================\n\n')
            fa.write('.. toctree::\n   :titlesonly:\n\n')
            fa.write('   宏定义<'+ module_part + '_define.rst>\n')
            fa.write('   类型定义<'+ module_part + '_typedef.rst>\n')
            fa.write('   枚举<'+ module_part + '_enum.rst>\n')
            fa.write('   结构体<' + module_part + '_struct.rst>\n')

def genrst_one_api(absfilename):
    print('genrst:', absfilename)
    md_file = absfilename
    tmp_name = absfilename[:-3]
    rst_file = tmp_name + '.rst'
    fd = open(md_file, 'r')
    fr = open(rst_file, 'w')
    if tmp_name.endswith('struct'):
        title = '结构体'
    elif tmp_name.endswith('enum'):
        title = '枚举'
    elif tmp_name.endswith('typedef'):
        title = '类型定义'
    elif tmp_name.endswith('define'):
        title = '宏定义'
    elif tmp_name.endswith('function'):
        title = '函数列表'
    else:
        fr.close()
        fd.close()
        return
    if conf.language == 'en':
        title = en_title[title]
    fr.write(title + '\n===================\n\n')
    fr.write('.. toctree::\n   :titlesonly:\n\n')
    module_part = os.path.basename(absfilename).split('_')[0]
    file_dir = os.path.dirname(absfilename)
    lib_dir = os.path.join(file_dir, 'lib')
    if not os.path.exists(lib_dir):
        os.mkdir(lib_dir)
    while True:
        line = fd.readline()
        if not line:
            break
        line = line.strip('\n')
        if line.endswith('.rst)'):
            line_name = line.split('](')[0].split('[')[1]#函数名
            line_file = line.split('](')[1].strip(')')#对应的rst文件
            struct_filename_dic[line_name] = 'exhalerst/lib/' + line_file
            fr.write('   ' + line_name + '<lib/' + line_file + '>\n')
            old_path = os.path.join(file_dir, line_file)
            new_path = os.path.join(lib_dir, line_file)
            shutil.copy(old_path, new_path)
    fd.close()
    fr.close()
    data_rst = os.path.join(file_dir, module_part + '_annotated.rst')
    if conf.language == 'en':
        with open(data_rst, 'w') as fa:
            fa.write('Data Structure\n===================\n\n')
            fa.write('.. toctree::\n   :titlesonly:\n\n')
            fa.write('   Define<'+ module_part + '_define.rst>\n')
            fa.write('   Typedef<'+ module_part + '_typedef.rst>\n')
            fa.write('   Enum<'+ module_part + '_enum.rst>\n')
            fa.write('   Struct<' + module_part + '_struct.rst>\n')
    else:
        with open(data_rst, 'w') as fa:
            fa.write('数据结构\n===================\n\n')
            fa.write('.. toctree::\n   :titlesonly:\n\n')
            fa.write('   宏定义<'+ module_part + '_define.rst>\n')
            fa.write('   类型定义<'+ module_part + '_typedef.rst>\n')
            fa.write('   枚举<'+ module_part + '_enum.rst>\n')
            fa.write('   结构体<' + module_part + '_struct.rst>\n')

def getRelativepath(source_abs_dir, target_abs_path):
    abs_list = re.findall('/[^/]+', source_abs_dir)
    if not abs_list:
        print("     ")
        sys.exit(0)
    dir_str = ''
    list_len = len(abs_list)
    i = 0
    relative_path = ''
    for onedir in abs_list:
        isfind = target_abs_path.find(os.path.join(dir_str, onedir))
        if isfind < 0:
            sfilename = re.sub(dir_str, '', target_abs_path)
            relative_path = os.path.join('../'*(list_len - i), sfilename[1:])
            break
        i = i + 1
        if dir_str == '':
            dir_str = onedir
        else:
            dir_str = os.path.join(dir_str, onedir[1:])
    return relative_path


def onefilerocesss(absfilename, target_abs_path):
    print('processing ' + absfilename)
    if absfilename == None:
        return None

    with open(absfilename, 'r') as fd:
        data = fd.read()
        #data = data.rstrip('\r\n')
        # gxdocref process
        gxdocreflist = re.findall('gxdocref[ \t]+\S+', data, flags = re.ASCII)
        for gxdocref in gxdocreflist:
            _struct = re.sub('gxdocref *', '', gxdocref)
            if _struct in struct_filename_dic.keys():
                sfilename = struct_filename_dic[_struct]
                sfilename = os.path.abspath(sfilename)
                abs_dir = os.path.dirname(absfilename)
                relative_path = getRelativepath(abs_dir, sfilename)
                rep = '[**' + _struct + '**](' + relative_path + ')'
                data = re.sub(gxdocref, rep, data)

        reflist = re.findall("\\\\ref\s+\S+", data, flags = re.ASCII) #为什么需要四个'\'
        for gxdocref in reflist:
            _struct = re.sub('\\\\ref *', '', gxdocref)
            if _struct in struct_filename_dic.keys():
                sfilename = struct_filename_dic[_struct]
                sfilename = os.path.abspath(sfilename)
                abs_dir = os.path.dirname(absfilename)
                relative_path = getRelativepath(abs_dir, sfilename)
                rep = '[**' + _struct + '**](' + relative_path + ')'
                data = re.sub('\\' + gxdocref, rep, data)

        style_list = re.findall('\[\*\*\S+?\*\*\] *\(\S+?\)', data, flags = re.ASCII)
        for style in style_list:
            _struct = re.sub('\[\*\*(\S+)\*\*\].+', '\g<1>', style)
            _struct = re.sub('\\_', '_', _struct)
            if _struct in struct_filename_dic.keys():
                sfilename = struct_filename_dic[_struct]
                sfilename = os.path.abspath(sfilename)
                abs_dir = os.path.dirname(absfilename)
                relative_path = getRelativepath(abs_dir, sfilename)
                rep = '[**' + _struct + '**](' + relative_path + ')'
                #print(style, rep, data.find(style))
                #style = '\[\*\*GxDmxSetSource\*\*\]\(group__autotest_1gafb6123a7202108566823f263e3318f74\.md\)'
                style = re.sub('\[', '\\\[', style)
                style = re.sub('\]', '\\\]', style)
                style = re.sub('\*', '\\\*', style)
                style = re.sub('\(', '\\\(', style)
                style = re.sub('\)', '\\\)', style)
                style = re.sub('\.', '\\\\.', style)
                data = re.sub(style, rep, data)
    with open(absfilename, 'w') as fd:
        fd.write(data)

def mdprocesss(md_dir, rst_dir):
    excluded_dirs = ['_build', 'copyinclude', 'doxyoutput', 'exhalerst', '__pycache__', 'scripts', '_static', '_templates']
    for dirpath,dirnames,filenames in os.walk(md_dir):
        dirnames[:] = [d for d in dirnames if d not in excluded_dirs]

        for filename in filenames:
            if filename.endswith('.md'):
                absfilename = os.path.abspath(os.path.join(dirpath,filename))
                onefilerocesss(absfilename, rst_dir)

def clean_mdfile(rootdir):
    for i in os.listdir(rootdir):
        path = os.path.join(rootdir, i)
        libpath = os.path.join(rootdir, 'lib', i)
        if path.endswith('md'):
            os.system('rm ' + path)
        if os.path.exists(libpath):
            os.system('rm ' + path)

def genmodule(app):
    rootdir = 'exhalerst'
    dic_info = getheadfile_datastructinfo(rootdir)
    module_dic = genmoduleindex(rstinfo.project_info, dic_info)
    getindexmdbymoduledic(module_dic, rootdir)
    rst_dir = ''
    bookdir = './'
    mdprocesss(bookdir, rst_dir)
    app.struct_filename_dic = struct_filename_dic
    clean_mdfile(rootdir)


def setup(app):
    app.setup_extension("exhale")
    app.connect("builder-inited", genmodule)
    return {
        'version': __version__,
    }

if __name__ == "__main__":
    genmodule(1)



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys
import time
sys.path.append(os.path.abspath("./scripts/_ext"))
sys.path.append(os.path.abspath("./"))
# sys.path.insert(0, os.path.abspath('.'))
import gxdocconfig


# -- Project information -----------------------------------------------------

#project = time.strftime('%Y-%m-%d %H:%M:%S')
project = "SOC"
copyright = '解释权归杭州国芯所有'
author = 'NationalChip'

# The full version, including alpha/beta/rc tags
release = 'v1.0'


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
#extensions = ['recommonmark', 'sphinx_markdown_tables']
extensions = gxdocconfig.extensions
extensions.extend([])
                    

one_page_one_api = 'yes'
no_build_dir = []
no_numbered_chapter = ['前言']

#html_js_files = [
#            'https://code.jquery.com/jquery-3.6.0.min.js',
#            ]
#html_js_files.append('singlehtml_nav.js')


#extensions = [
#        'breathe',
#        'exhale',
#        'myst_parser',
#        'genmodule',
##        'recommonmark'
#
#]
    
breathe_projects = {
        project : "./doxyoutput/XML",
        }

breathe_default_project = project

exhale_args = {
        "containmentFolder": "./exhalerst",
        "rootFileName":          "index.rst",
        "rootFileTitle":         project + " API",
        "doxygenStripFromPath":  "..",
        # Suggested optional arguments
        "createTreeView":        True,
        # TIP: if using the sphinx-bootstrap-theme, you need
        # "treeViewIsBootstrap": True,
        "exhaleExecutesDoxygen": False,
        #"exhaleDoxygenStdin":    "INPUT = ../include"
        "verboseBuild": False,
        "generateBreatheFileDirectives": False,
        #"rootFileTitle":"rootFileTitle",
        #"afterTitleDescription":"afterTitleDescription",
        #"afterHierarchyDescription":"afterHierarchyDescription",
        #"fullApiSubSectionTitle":"fullApiSubSectionTitle",
        #"afterBodySummary":"afterBodySummary",

        }

#myst_heading_slug_func 当跳转的锚点有问题时需要实现这个函数，接收一个字符串同时 返回一个字符串
myst_heading_anchors = 5 # h1-h5的标题都可以作为锚点
myst_enable_extensions = [
        "amsmath",
        "colon_fence",
        "deflist",
        "dollarmath",
        "html_admonition",
        "html_image",
        "attrs_image",
        #"linkify",
        "replacements",
        "smartquotes",
        "substitution",
        "tasklist",
        ]

# Tell sphinx what the primary language being documented is.
primary_domain = 'c'

# Tell sphinx what the pygments highlight language should be.
highlight_language = 'c'

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
#language = 'zh_CN'
language = 'zh'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
#html_theme = 'alabaster'
#html_theme = 'piccolo_theme'
html_theme = 'sphinx_rtd_theme'
#html_theme = 'sphinxawesome_theme'
# all theme : https://sphinx-themes.org
# rtd_theme document: https://sphinx-rtd-theme.readthedocs.io/en/latest/configuring.html
html_theme_options = {
    #'analytics_id': 'G-XXXXXXXXXX',  #  Provided by Google in your dashboard
    #'analytics_anonymize_ip': False,
    #'logo_only': False,
    #'display_version': True,
    #'prev_next_buttons_location': 'bottom',
    #'style_external_links': False,
    #'vcs_pageview_mode': '',
    #'style_nav_header_background': 'white',
    ## Toc options
    #'collapse_navigation': True,
    #'sticky_navigation': True,
    'navigation_depth': -1,
    #'includehidden': True,
    #'titles_only': False
}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

source_suffix = {
        '.rst': 'restructuredtext',
        '.txt': 'markdown',
        '.md': 'markdown',
        }

master_doc = gxdocconfig.master_doc


latex_engine = 'xelatex'

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
     'fncychap': r'\usepackage[Lenny]{fncychap}',
     'papersize': 'a4paper',
     'classoptions': ',oneside',

    # The font size ('10pt', '11pt' or '12pt').
    #
     'pointsize': '10pt',
     'maxlistdepth': '99',
     'preamble':r'''

        \usepackage{fancyhdr}
        \pagestyle{fancy}
        \usepackage{etoolbox}
        \patchcmd{\chapter}{\thispagestyle{plain}}{\thispagestyle{fancy}}{}{}
        \hypersetup{
            colorlinks=true,
            linkcolor=blue,
            citecolor=blue,
            filecolor=blue,
            urlcolor=blue,
            %%pdfborder={0 0 0}
        }
        \usepackage{longtable}
        \usepackage{etoolbox}
        \AtBeginEnvironment{longtable}{\footnotesize}
        \let\origtabular\longtable
        \let\endorigtabular\endlongtable
        \let\origtabulary\longtable
        \let\endorigtabulary\endlongtable
        \usepackage{caption}
        \usepackage{draftwatermark}
        \SetWatermarkText{Nationalchip} % the Text
        \SetWatermarkLightness{0.9} % the lightness from 0 to 1, default 0.8
        \SetWatermarkScale{1.0} % the scale, default 1.2
        \captionsetup[figure]{name=}
        \captionsetup[table]{name=}
        \usepackage{tocloft}
        \cftsetindents{figure}{1.5em}{3.5em}
        \cftsetindents{table}{1.5em}{3.5em}
        \usepackage{etoolbox}
        \usepackage{xstring}
        \usepackage{minitoc}
        \usepackage{tabulary}
        \usepackage{tabularx}
        \usepackage{chngcntr}
        \usepackage{fourier}
        \AtBeginEnvironment{tabular}{\small}
        \AtBeginEnvironment{tabularx}{\small}
        \AtBeginEnvironment{tabulary}{\small}
        \AtBeginEnvironment{longtable}{\small}
        \AtBeginEnvironment{xltabular}{\small}
        \usepackage{graphicx}
        \usepackage{titling}
        \title{HIMPP V4.0 媒体处理软件}
        \author{文档版本\hspace{4em}04}
        \date{发布日期\hspace{4em}2022-05-29}
        \hypersetup{pdftitle={SOC CHIP TEST}, pdfauthor={gx test}}

        \begin{document}
        \begin{titlepage}
        \centering
        \includegraphics[width=0.4\paperwidth]{fengmian.png}\par\vspace{1cm}
        \vfill
        \raggedright{\scshape\LARGE\thetitle\par}
        %%\vspace{1.5cm}
        \raggedright{\huge\bfseries 开发参考\par}
        \vspace{2cm}
        \raggedright{\large\theauthor\par}
        \raggedright{\large\thedate\par}
        \vspace{4cm}
        \end{titlepage}

     '''

    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, project + '.tex', project + ' manual Documentation',
     'liufei', 'manual', False),
]





from __future__ import unicode_literals
import os
import sys
import re
import pickle
from sphinx import addnodes
from docutils import nodes

__version__ = "1.0.0"

def deal_ref(target, docname, app):
    ref_text = re.sub('\\\\ref *', '', target)
    if "struct_filename_dic" in dir(app) and\
            ref_text in app.struct_filename_dic:
        ref_path = app.struct_filename_dic[ref_text][6:-4]
    else:
        return 0
    refnode = addnodes.pending_xref('',
            refdoc = docname, #当前文件
            refdomain='std',
            refexplicit=False,
            reftype='doc',
            reftarget= ref_path,
            )
    #inner_node = nodes.inline("", ref_text, classes=["xref", "myst"])
    inner_node = nodes.inline("", ref_text, classes=["xref", "std", "std-doc"])
    refnode.append(inner_node)
    return refnode

def deal_markdown_ref(target, docname):
    ref_text = re.findall('\[.*\]' ,target)[0][1:-1] # [链接名称]
    ref_path = re.findall('\(.*#.*\)' ,target)[0][1:-1]  #  (md#title)
    refnode = addnodes.pending_xref('',
            refdoc = docname, #当前文件
            #refdomain='std',
            #refexplicit=False,
            refdomain=True,
            refexplicit=True,
            reftype='myst',
            reftarget= ref_path,
            )
    inner_node = nodes.inline("", ref_text, classes=["xref", "myst"])
    refnode.append(inner_node)
    return refnode

def deal_txt(data):
    return nodes.Text(data, data)

def ref_node(node, docname, app):
    for i in range(len(node.children)):
        child = node.children[i]
        data = child.rawsource
        sections = re.split('(\[.*?\]\(.*?#.*?\)|\\\\ref\s+\S+)', data)
        if len(sections) > 1:
            child.pop(0)
            for section in sections:
                if re.findall('\[.*?\]\(.*?#.*?\)', section):
                    child.append(deal_markdown_ref(section, docname))
                elif re.findall('\\\\ref\s+\S+', section):
                    ret = deal_ref(section, docname, app)
                    if ret != 0:
                        child.append(ret)
                    else:
                        child.append(deal_txt(section))
                else:
                    child.append(deal_txt(section))
        elif len(child.children) != 0:
            ref_node(child, docname, app)

def get_rstTable(rst_file):
    rst_doctree = rst_file[:-3] + "doctree"
    with open(rst_doctree, 'rb') as ft:
        t_doctree = pickle.load(ft)
        table = t_doctree.children[0]
    return table

def show_node(node, path):
    for i in range(len(node.children)):
        child = node.children[i]
        text = child.rawsource
        if text.find('gxtableref') >= 0:
            rstTable_file = re.findall('0000gxtableref.*rst', text)[0]
            rstTable_file = path + '/' + rstTable_file
            table_doctree = get_rstTable(rstTable_file)
            node.pop(i)
            node.insert(i,table_doctree)
        elif len(child.children) != 0:
            show_node(child, path)

def doctree_read(app, doctree):
    file_source = doctree.document['source']
    if os.path.basename(file_source).find('0000gxtableref') >= 0:
        file_path = file_source.split('/_output/')[1]
        doc_dir = os.path.dirname(file_path)
        md_file = '_'.join(os.path.basename(file_path).split('_')[1:-1])
        docname = os.path.join(doc_dir, md_file)
        ref_node(doctree, docname, app)
    else:
        doctree_path = '_build/doctrees/' + os.path.dirname(file_source.split('/_output/')[1])
        show_node(doctree, doctree_path)

def setup(app):
    app.connect("doctree-read", doctree_read)
    return {
        'version': __version__,
    }



#!/usr/bin/env python3

import os
import sys
import pdfkit
sys.path.append('../../')
import conf

import PyPDF2
from PIL import Image

def get_image_size(image_path):
    with Image.open(image_path) as img:
        return img.size

def get_pdf_dimensions(pdf_file):
    with open(pdf_file, 'rb') as file:
        pdf_reader = PyPDF2.PdfReader(file)
        page = pdf_reader.pages[0]  # 获取第一页，可以更改页码以获取其他页面的尺寸
        width = page.mediabox.width
        height = page.mediabox.height
        return width, height

def get_coeffi(w, h):
    print(500/w * h)
    if 500/w * h >  700: #宽度放大时，高度是否超范围
        h_coeffi = 700 /h
        print(h_coeffi)
        if h_coeffi  > 2:
            h_coeffi = 1
        coeffi = h_coeffi*w / 500
    else:
        #coeffi = 460/w
        coeffi = 1
    return str(round(coeffi, 1))

#def get_coeffi(w, h):
#    x = 500
#    y = 700
#    if w < x and h < y: #图片在页面范围内，不做缩放
#        coeffi = w/x
#    elif h/w >= y/x: #图片超过页面范围，且更窄，以h进行缩放
#        h_coeffi = y/h
#        coeffi = w*h_coeffi/x
#    elif h/w < y/x: #图片超过页面范围，且更宽，以w进行缩放
#        coeffi = x/w
#    return str(round(coeffi, 1))

#def get_coeffi(w, h):
#    x = 500 #pdf有效显示宽度, A4大小
#    y = 700 #pdf有效显示高度, A4大小
#
#    print(">>>>>>>>>>>>", w, h, w/h)
#    print(w/h >= 1.2)
#    if h/w >= y/x: #图片较窄，以h进行缩放
#        if h<y:
#            # h<pdf高度，无需缩放
#            coeffi = w/x
#        else:
#            # h>pdf高度，将h缩小至y
#            h_coeffi = y/h
#            coeffi = w*h_coeffi/x
#    elif h/w < y/x: #图片较宽，以w进行缩放
#        coeffi = 1
#        #if w<x:
#        #    # w<pdf宽度
#        #    if w/h >= 1.2:
#        #        # 图片较扁，拉长至x
#        #        coeffi = 1
#        #    else:
#        #        # 图片较方，不缩放
#        #        coeffi = w/x
#        #else:
#        #    # w>pdf宽度, 将w缩小至x
#        #    coeffi = 1
#    print(str(round(coeffi, 1)))
#    return str(round(coeffi, 1))

def main():
    in_file = sys.argv[1]
    new_file = "new_file.tex"
    pdfkit_options = {
        'encoding':'UTF-8'
    	}

    def check_pdf_tex(pdf_tex):
        new_pdf_tex = 'new_' + pdf_tex
        ft = open(pdf_tex, 'r')
        fn = open(new_pdf_tex, 'w')
        while 1:
            line = ft.readline()
            if line.find('_') >= 0 and line.find('\includegraphics') <= 0:
                fn.write(line.replace('_', '\\_').replace('&', '\\&'))
            elif line.find('&') >= 0 :
                fn.write(line.replace('&', '\\&'))
            else:
                fn.write(line)
            if not line:
                break
        ft.close()
        fn.close()
        mv_cmd = 'mv ' + new_pdf_tex + ' ' + pdf_tex
        os.system(mv_cmd)
    
    fi = open(in_file, 'r')
    fo = open(new_file, 'w')
    table_start = 0
    longtable_start = 0
    longtable_tmp = 0
    title_start = 0
    table_width = ''
    get_width = 0
    chapter_name = ''
    while 1:
        line = fi.readline()
        #if line.find('&') >= 0 :
        #    fo.write(line.replace('&', '\\&'))
        if line.find('\\sphinxattablestart') >= 0:
            table_start = 1
        if line.find('\\sphinxattableend') >= 0:
            table_start = 0
        if line.startswith('\\sphinxincludegraphics') and \
                line.find('.svg') >= 0 and line.find('.drawio') < 0:
            # SVG图片，不是drawio
            reg_name = line.split('{{')[1].split('}')[0]
            if not os.path.exists(reg_name + '.svg'):
                continue
            svg_cmd = "inkscape -D -z --file=" + reg_name + ".svg --export-pdf=" + \
                    reg_name + ".pdf --export-latex"
            print(reg_name)
            os.system(svg_cmd)
            width, height = get_pdf_dimensions(reg_name + '.pdf')
            coeffi = get_coeffi(width, height)
            check_pdf_tex(reg_name + '.pdf_tex')
    
            ##msg = '\\FloatBarrier\n'
            #msg += '\\begin{figure}[htbp]\n\\centering\n\\def\\svgwidth{' + coeffi + '\\columnwidth}\n'
            #msg += '\\begin{figure}[h]\n\\centering\n\\def\\svgwidth{' + coeffi + '\\columnwidth}\n'
            msg = '\\begin{figure}[H]\n\\centering\n\\def\\svgwidth{' + coeffi + '\\columnwidth}\n'
            msg += '\\footnotesize\n'
            msg += '\\input{' + reg_name + '.pdf_tex}\n'
            msg += '\\caption{' + reg_name.replace("_", "\\_") + '}\n'
            msg += '\\end{figure}\n'
            ##msg += '\\FloatBarrier\n'
            fo.write(msg)
            print(line, end = '')
        elif line.find('\\sphinxincludegraphics') >= 0 and \
                line.find('.svg') >= 0 and line.find('.drawio') >= 0:
            # SVG图片，是drawio
            svg_path = line.split('sphinxincludegraphics')[1].split('{{')[1].split('}')[0]
            svg_name = os.path.basename(svg_path)
            new_svg_name = svg_name.replace('.drawio', '')
            if not os.path.exists(svg_name + '.svg'):
                continue
            os.system("cp " + svg_name + '.svg ' + new_svg_name + '.svg')
            drawio_to_pdf_cmd = "xvfb-run -a -s '-screen 0 1280x1024x24' drawio -x --width 500 -f pdf " + new_svg_name +'.svg'
            os.system(drawio_to_pdf_cmd)
            os.system('pdfcrop "'  + new_svg_name + '.pdf"')
            os.system('mv "'  + new_svg_name + '-crop.pdf" ' + '"'  + new_svg_name + '.pdf"')

            width, height = get_pdf_dimensions(new_svg_name + '.pdf')
            coeffi = get_coeffi(width, height)
    
            ##msg = '\\FloatBarrier\n'
            #msg += '\\begin{figure}[htbp]\n\\centering\n'
            #msg += '\\begin{figure}[h]\n\\centering\n'
            msg = '\\begin{figure}[H]\n\\centering\n'
            msg += '\\footnotesize\n'
            msg += '\\includegraphics[width=' + coeffi + '\\textwidth]{' + new_svg_name + '.pdf}\n'
            msg += '\\caption{' + new_svg_name.replace("_", "\\_") + '}\n'
            msg += '\\end{figure}\n'
            ##msg += '\\FloatBarrier\n'
            fo.write(msg)
            print(line, end = '')
        elif line.startswith('\\sphinxincludegraphics'):
            # 其他图片，如png等
            image_path = line.split('sphinxincludegraphics')[1].split('{{')[1].split('}')[0]
            image_type = line.split('sphinxincludegraphics')[1].strip('{}\n').split('.')[-1]
            image_name = os.path.basename(image_path)
            image_msg = line.split('sphinxincludegraphics')[1]

            width, height = get_image_size(image_path + '.' + image_type)
            coeffi = get_coeffi(width, height)

            ##msg = '\\FloatBarrier\n'
            #msg += '\\begin{figure}[htbp]\n\\centering\n'
            #msg += '\\begin{figure}[h]\n\\centering\n'
            msg = '\\begin{figure}[H]\n\\centering\n'
            msg += '\\footnotesize\n'
            #msg += '\\includegraphics[width=' + coeffi + '\\textwidth]{' + new_svg_name + '.pdf}\n'
            msg += '\\includegraphics[width=' + coeffi + '\\textwidth]' + image_msg + '\n'
            #msg += '\\includegraphics' + image_msg
            msg += '\\caption{' + image_name.replace("_", "\\_") + '}\n'
            msg += '\\end{figure}\n'
            ##msg += '\\FloatBarrier\n'
            fo.write(msg)
        elif line.startswith('\\sphinxstylestrong'):
            fo.write(line)
            fo.write('\\\\' +  '\n')
        elif line.find('0000gxhtmltableref') >= 0:
            print(line)
            html_file = line.strip('\n').replace('\\','')
            print(html_file)

            pdf_file = os.path.basename(html_file)[:-5] + '.pdf'
            pdfkit.from_file(html_file, pdf_file, options = pdfkit_options)
            #os.system('pdfcrop ' + pdf_file)
            #crop_pdf = pdf_file[:-4] + '-crop.pdf'
            #os.system('mv ' + crop_pdf + ' ' + pdf_file)

            width, height = get_pdf_dimensions(pdf_file)
            coeffi = get_coeffi(width, height)
            print(">>>>>>>..", coeffi)
            print(">>>>>>>..", width, height)

            msg = '\\begin{center}\n'
            #msg += '\\includegraphics{' + pdf_file + '}\n'
            msg += '\\includegraphics[width=' + coeffi + '\\textwidth]{' + pdf_file + '}\n'
            msg += '\\end{center}\n'
            fo.write(msg)
        elif line.find('gxpdfref') >= 0 and line.find('sphinxhref') >= 0:
            line = line.replace('\\sphinxhref', '')
            section = line.split('gxpdfref')
            new_line = ''
            for sec in section:
                if sec.find('}{') >= 0:
                    tmp = sec.split('}')
                    link_path = tmp[0]
                    link_name = tmp[1]
                    txt = tmp[2] 
                    new_line +=  "\hyperref[\detokenize{" + link_path + "}]{\sphinxcrossref{\DUrole{std,std-doc}" + link_name + "}}}}" + txt
                else:
                    new_line += sec
            msg = new_line
            fo.write(msg)
        elif line.startswith('\\usepackage{xeCJK}'):
            msg = line
            #msg += '\\usepackage{hyperref}\n'
            msg += '\\usepackage[numbered]{bookmark}\n'
            msg += '\\usepackage[section]{placeins}\n'
            msg += '\\usepackage{setspace}\n'
            msg += '\\usepackage{tabularx}\n'
            msg += '\\usepackage{xltabular}\n'
            msg += '\\usepackage{float}\n'
            #msg += '\\newlength{\customtablewidth}'
            #msg += '\\setlength{\customtablewidth}{1\linewidth}'
            msg += '\\usepackage{array}\n'
            msg += '\\usepackage{seqsplit}\n'
            msg += '\\newcolumntype{Y}[1]{>{\\hsize=#1\\hsize\\seqsplit}X}\n'
            msg += '\\usepackage{makecell}\n'
            msg += '\\usepackage{longtable}\n'
            msg += '\\usepackage[table]{xcolor}\n'
            msg += '\\usepackage{graphicx}\n'
            msg += '\\renewcommand\\arraystretch{1.5}\n'
            msg += '\\renewcommand\\baselinestretch{1.5}\n'
            fo.write(msg)
        #elif line.startswith('\\chapter{前言}'):
        #    print(conf.no_numbered_chapter)
        #    sys.exit()
        #    msg = '\\chapter*{前言}\n'
        #    msg += '\\addcontentsline{toc}{chapter}{前言}\n'
        #    fo.write(msg)
        elif line.startswith('\\chapter{'):
            chapter_name = line.split('chapter{')[1].split('}')[0]
            if chapter_name in conf.no_numbered_chapter : 
                msg = '\\chapter*{' + chapter_name + '}\n'
                msg += '\\addcontentsline{toc}{chapter}{' + chapter_name + '}\n'
                fo.write(msg)
            else:
                msg = line
                msg += '\\minitoc\n'
                msg += '\\minilof\n'
                msg += '\\minilot\n'
                fo.write(msg)
        elif line.startswith('\\section'):
            if chapter_name in conf.no_numbered_chapter : 
                section_name = line.split('section{')[1].split('}')[0]
                msg = '\\section*{' + section_name + '}\n'
                msg += '\\addcontentsline{toc}{section}{' + section_name + '}\n'
                fo.write(msg)
            else:
                msg = line
                fo.write(msg)
        elif line.startswith('\\subsection'):
            if chapter_name in conf.no_numbered_chapter : 
                section_name = line.split('subsection{')[1].split('}')[0]
                msg = '\\subsection*{' + section_name + '}\n'
                msg += '\\addcontentsline{toc}{subsection}{' + section_name + '}\n'
                fo.write(msg)
            else:
                msg = line
                fo.write(msg)
        elif line.startswith('\\subsubsection'):
            if chapter_name in conf.no_numbered_chapter : 
                section_name = line.split('subsubsection{')[1].split('}')[0]
                msg = '\\subsubsection*{' + section_name + '}\n'
                msg += '\\addcontentsline{toc}{subsubsection}{' + section_name + '}\n'
                fo.write(msg)
            else:
                msg = line
                fo.write(msg)
        elif line.startswith('\\subsubsubsection'):
            if chapter_name in conf.no_numbered_chapter : 
                section_name = line.split('subsubsubsection{')[1].split('}')[0]
                msg = '\\subsubsubsection*{' + section_name + '}\n'
                msg += '\\addcontentsline{toc}{subsubsubsection}{' + section_name + '}\n'
                fo.write(msg)
            else:
                msg = line
                fo.write(msg)
        elif line.startswith('\\phantomsection\\label'):
            msg = line
            msg += '\\lhead{\includegraphics[width=1cm]{icon.png}}\n'
            fo.write(msg)
        elif line.find('\\sphinxtablecontinued') >= 0 and line.find('续上页') >= 0:
            msg = '{\\makebox[0pt]{\\sphinxtablecontinued{续上页}}}\\\\'
            fo.write(msg)
        elif line.startswith('\\title{'):
            title_start = 1
        elif title_start == 1:
            if line.startswith('\\sphinxtableofcontents'):
                title_start = 0
                msg = '\\counterwithin{figure}{chapter}\n'
                msg += '\\counterwithin{table}{chapter}\n'
                msg += '\\renewcommand{\\thefigure}{图\\thechapter-\\arabic{figure}}\n'
                msg += '\\renewcommand{\\thetable}{表\\thechapter-\\arabic{table}}\n'
                msg += '\\pdfbookmark[1]{Contents}{toc}\n'
                msg += '\\dominitoc\n'
                msg += '\\dominilof\n'
                msg += '\\dominilot\n'
                msg += '\\tableofcontents\n'
                msg += '\\listoffigures\n'
                msg += '\\listoftables\n'
                fo.write(msg)
            else:
                continue
        elif table_start == 1 and line.startswith('\\begin{tabulary}'):
            if get_width:
                msg = "\\begin{xltabular}{\\linewidth}{|"
                print('table_start>>>', line)
                for t in range(len(line.strip('\n').split('[t]')[-1].strip('{}|').split('|'))):
                    msg += 'Y{' + table_width[t] + '}|'
                fo.write(msg + '}\n')
                get_width = 0
                table_width = ''
            else:
                msg = "\\begin{xltabular}{\\linewidth}"
                #msg = "\\begin{xltabular}{\\customtablewidth}"
                msg += line.split('[t]')[-1].replace('T', 'l')
                #msg += line.split('[t]')[-1].replace('T', 'X')
                msg = msg.replace('l|}', 'X|}')
                fo.write(msg)
        elif table_start == 1 and line.startswith('\\end{tabulary}'):
            msg = "\\caption{ }\n"
            msg += "\\end{xltabular}"
            fo.write(msg)
        #elif line.startswith('\\begin{savenotes}\\sphinxatlongtablestart\\begin{longtable}'):
        elif line.startswith('\\begin{longtable}'):
            longtable_start = 1
            if get_width:
                msg = "\\begin{xltabular}{\\linewidth}{|"
                print('table_start>>>', line)
                for t in range(len(line.strip('\n').split('[c]')[-1].strip('{}|').split('|'))):
                    msg += 'Y{' + table_width[t] + '}|'
                fo.write(msg + '}\n')
                get_width = 0
                table_width = ''
            else:
                msg = line.split('\\begin{longtable}')[0] + '\n'
                msg += "\\begin{xltabular}{\\linewidth}"
                msg += line.split('[c]')[-1]
                msg = msg.replace('l|}', 'X|}')
                fo.write(msg)
        elif line.startswith('\\sphinxtableatstartofbodyhook') and longtable_start == 1:
            fo.write(line)
            table_start = 1
        #elif line.startswith('\\end{longtable}\\sphinxatlongtableend\\end{savenotes}'):
        elif line.startswith('\\end{longtable}'):
            table_start = 0
            longtable_start = 0
            msg = "\\caption{ }\n"
            msg += "\\end{xltabular}"
            msg += line.split('{longtable}')[-1]
            fo.write(msg)
        elif table_start == 1 and line.find(";") >= 0:
            #msg = '\\makecell[l]{' + line.replace(';', '\\\\') + '}'
            msg = line.strip(';').replace(';', '\\newline ')
            if table_start == 1 and not line.startswith(('\\', '&', '\n')) \
                    and line.find('\\sphinx') < 0:
                msg = '\\seqsplit{' + msg[:-1] + '}\n'
            fo.write(msg)
        elif table_start == 1 and not line.startswith(('\\', '&', '\n')) \
                and line.find('\\sphinx') < 0:
            msg = '\\seqsplit{' + line[:-1] + '}\n'
            fo.write(msg)
        elif table_start == 1 and line == '\n':
            msg = '\\quad\n'
            fo.write(msg)
        elif line.startswith('gxtable\\_col\\_width\\_percent'):
            table_width = line[:-4].replace('gxtable\\_col\\_width\\_percent:{[}', '').split(',')
            get_width = 1
            print(line, table_width)
        elif line.startswith('\\multicolumn'):
            file_position = fi.tell()
            next_line = fi.readline()
            if next_line.find('\\sphinxtablecontinued') >= 0:
                next_line = fi.readline()
            else:
                fo.write(line)
                fi.seek(file_position)
        elif line.find('\&nbsp;') >= 0  and table_start == 1:
            fo.write(line.replace('\&nbsp;', ''))
        elif line.find('\&nbsp;') >= 0  and table_start == 1:
            fo.write(line.replace('\&nbsp;', ''))
        elif line.find('‣') >= 0:
            if table_start == 1:
                fo.write(line.replace('‣', '\\newline'))
            else:
                fo.write(line.replace('‣', '\\\\'))
        else:
            fo.write(line)
        if not line:
            break
    fi.close()
    fo.close()
    mv_cmd = 'mv "' + new_file + '" "' + in_file + '"'
    print(mv_cmd)
    os.system(mv_cmd)

if __name__ == "__main__":
    main()



"""
Documentation
-------------
to be continue

"""

from setuptools import setup, find_packages

long_description = __doc__

def main():
    setup(
        name="gxddk",
        description="sphinx doc build",
        keywords="sphinx doc",
        long_description=long_description,
        version="1.2.6",
        author="zhaobk",
        author_email="zhaobk@nationalchip.com",
        packages=find_packages(),
        include_package_data=True,
        data_files=[
            ('scripts/_templates', [
                'gxddk/scripts/_templates/layout.html',
                'gxddk/scripts/_templates/search.html',
                ]),
            ('scripts/_static', [
                'gxddk/scripts/_static/style.css',
                ]),
            ('scripts/_ext', [
                'gxddk/scripts/_ext/genmodule.py',
                'gxddk/scripts/_ext/retex.py',
                'gxddk/scripts/_ext/rst_table.py',
                ]),
            ('scripts', [
                'gxddk/scripts/envexec',
                ]),
            ],
        entry_points={
            'console_scripts':[
                'gxddkbuild=gxddk.gxddkbuild:run',
                ]
            }
    )


if __name__ == "__main__":
    main()
